/** Types and constants used by the terminal interface. 
    @author Stuart D. Gathman
    Copyright (C) 1998 Business Management Systems, Inc.
 */
class PSConstants {
  typedef unsigned short CHTYPE;	// character attribute pair
  enum dir { DIR_UP, DIR_DOWN, DIR_LEFT, DIR_RIGHT, DIR_NONE };
  enum {
    A_STANDOUT	= 0100000,
    A_UNDERLINE	= 0000400,
    A_REVERSE	= 0001000,
    A_BLINK	= 0002000,
    A_DIM		= 0004000,
    A_BOLD	= 0010000,
    A_ALTCHARSET	= 0000200,

    /* The next two are subject to change so don't depend on them */
    A_INVIS	= 0020000,
    A_MODIFIED	= 0040000,
    A_PROTECT	= A_DIM,

    A_NORMAL	= 0000000,
    A_ATTRIBUTES	= 0177600,
    A_CHARTEXT	= 0000177
  };

/**
	line drawing character codes
*/

  enum {
    ACS_BSSB	= 218,
    ACS_SSBB	= 192,
    ACS_BBSS	= 191,
    ACS_SBBS	= 217,
    ACS_SBSS	= 180,
    ACS_SSSB	= 195,
    ACS_SSBS	= 193,
    ACS_BSSS	= 194,
    ACS_BSBS	= 196,
    ACS_SBSB	= 179,
    ACS_SSSS	= 197,

    /*
     * Human readable names for the most commonly used characters.
     * "Upper", "right", etc. are chosen to be consistent with the vt100 manual.
     */

    ACS_URCORNER	= ACS_BBSS,
    /*			ACS_BSBB	*/
    ACS_HLINE	= ACS_BSBS,
    ACS_ULCORNER	= ACS_BSSB,
    ACS_TTEE	= ACS_BSSS,
    /*			ACS_SBBB	*/
    ACS_LRCORNER	= ACS_SBBS,
    ACS_VLINE	= ACS_SBSB,
    ACS_RTEE	= ACS_SBSS,
    ACS_LLCORNER	= ACS_SSBB,
    ACS_BTEE	= ACS_SSBS,
    ACS_LTEE	= ACS_SSSB,
    ACS_PLUS	= ACS_SSSS,
    ACS_S1	= 'o'|A_ALTCHARSET,	/* scan line 1 */
    ACS_S9	= 's'|A_ALTCHARSET,	/* scan line 9 */
    ACS_DIAMOND	= 4,			/* diamond */
    ACS_CKBOARD	= 177,                	/* checker board (stipple) */
    ACS_DEGREE	= 248,        		/* degree symbol */
    ACS_PLMINUS	= 241,			/* plus/minus */
    ACS_BULLET	= 249,			/* bullet */
	    /* Teletype 5410/5420 symbols */
    ACS_LARROW	= 27,			/* arrow pointing left */
    ACS_RARROW	= 26,			/* arrow pointing right */
    ACS_DARROW	= 25,			/* arrow pointing down */
    ACS_UARROW	= 24,			/* arrow pointing up */
    ACS_BOARD	= 178,			/* board of squares */
    ACS_LANTERN	= 21,			/* lantern symbol */
    ACS_BLOCK	= 219			/* solid square block */
  };

};
