#pragma interface
#include "PScreen.h"

/** A implementation of PScreen based on terminfo with no screen buffer.
    All output is directly implemented via available terminfo capabilities.
    This should work on AT&T and AIX terminfo.
 */
extern "C" int _outchar(int);

class PSTinfo: public PScreen {
  struct term *cur_term;
  struct _bool_struct *cur_bools;
  struct _num_struct *cur_nums;
  struct _str_struct *cur_strs;
  short lasttype;
  CHTYPE camode;
  struct myio;
  myio *mybuf;
  static myio *cur;
  friend int _outchar(int c);
  void mvcur(int oldrow,int oldcol,int row,int col);
public:
  PSTinfo();
  ~PSTinfo();
  void setTerm(const char *name);	// set terminal type
  void sync();
  /* write string with constant attribute */
  virtual void write(int row, int col, int cnt, const char *s, CHTYPE a);
  /* write array of different attributes */
  virtual void write(int row, int col, int cnt, const CHTYPE far *buf);
  /* fill rectangle with same char + attribute */
  virtual void fill(const Rect *, CHTYPE);
  /* slide rectangle, return false if not supported */
  virtual bool slide(const Rect *r, dir d, int cnt);

  virtual void curpos(int row, int col);	/* position cursor */

  /* set cursor type
     0 = invisible
     >0 varies with terminfo but visible
     return previous type
   */
  virtual int curtype(int type);

  /* argument is program to pipe ascii screen data into */
  virtual int prtscr(const char *prog);	/* print screen */

  /* stop displaying changes and rescan from the top */
  virtual void quit();

  /* argument says what kind of noise/flash to make,
     0 is "standard", other values terminfo dependent but do something
   */
  virtual void beep(int type = 0);

  /* load font for a character if terminal supports it */
  virtual bool font(CHTYPE , unsigned char [16]);

  /* send characters to attached printer, return false if unsupported */
  virtual bool PSprint(const char *buf, int cnt);

  virtual void dropscrn(); // return to roll mode
  virtual void restscrn(); // restore full screen mode and contents
  void insertLines(int);
  void deleteLines(int);
  int outchar(int);
  int putp(const char *);
};
