/* Physical screen interface.
   Author: Stuart D. Gathman
   Copyright 1988-1997 Business Management Systems, Inc.
   Loosely based on a book by Marc Rochind

 */

#include <Display.h>
#include <Attrib.h>

struct FILE;

class PScreen: public PSConstants {
  FILE *tracef;
  short savcurs;
protected:
  short prow,pcol,curtype;
  short PSrows, PScols;	/* physical screen size */
  /*  convert screen character to printable form */
  int toprt(CHTYPE);
public:
  PScreen();
  virtual ~PScreen();
  void trace(FILE *);
  virtual void sync() = 0;
  int getRows() const { return PSrows; }
  int getCols() const { return PScols; }

  /* write string with constant attribute */
  virtual void write(int row, int col, int cnt, const char *s, CHTYPE a) = 0;

  /* write array of different attributes */
  virtual void write(int row, int col, int cnt, const CHTYPE far *buf) = 0;

  /* fill rectangle with same char + attribute */
  virtual void fill(const Rect *, CHTYPE) = 0;

  /* slide rectangle, return false if not supported */
  virtual bool slide(const Rect *r, dir d, int cnt);

  /** position cursor */
  virtual void curpos(int row, int col) = 0;	

  /* set cursor type
     0 = invisible
     >0 varies with terminfo but visible
     return previous type
   */
  virtual int setType(int type);

  /* redraw screen, return false if not supported  */
  virtual bool refresh();	

  /* argument is program to pipe ascii screen data into */
  virtual int prtscr(const char *prog);	/* print screen */

  /* stop displaying changes and rescan from the top */
  virtual void quit();

  /* argument says what kind of noise/flash to make,
     0 is "standard", other values terminfo dependent but do something
   */
  virtual void beep(int type = 0);

  /** load font for a character if terminal supports it,
    return false if unsupported. */
  virtual bool font(CHTYPE , unsigned char [16]);

  /* send characters to attached printer, return false if unsupported */
  virtual bool PSprint(const char *buf, int cnt);

  virtual void dropscrn() = 0; // return to roll mode
  virtual void restscrn() = 0; // restore full screen mode and contents
};
