#include <inq.h>
#ifndef MORE
#include <port.h>
#endif
#ifndef FCBFLG_DEF
/*
   Things to remember when using fcb:

1. Before calling fcbbeg, your logical field data must be initialized
   to their "null" values (i.e., 0,blanks,zeroD,zeroM or the file equivalents).

2. If you are using viewio, any data that is not supposed to be displayed
   (but is part of any key) must still exist in your logical field table.
   You should put these "hidden" entrys in your field table below the last
   non-hidden entry and before the putmore entry (if you have a putmore).
   The "hidden" entry's field number should be set to the putmore entry's field
   number or, if you do not have a putmore entry, the first field after the
   table.  All other values except the data pointer should be set to 0.
   For example:
     
    static FIELD TBL[] = {
      strgen(0,0,putmore,0,"T",0),
      strgen(1,getstrn,putstrn,st.code,"RU",""),
      strgen(2,0,putstrn,st.desc,0,0),
      strgen(3,0,0,st.type,0,0),	hidden entry
      strgen(3,0,0,st.flags,0,0),	hidden entry
      strgen(3,0,putmore,0,"B",0)
    };

    Note: If you do not have bottom putmore entry nor any other fields
	  below the table, then you will have to put a dummy field in
	  your screen image file and use that field number for your
	  hidden fields.

3. You cannot have a pointer to the structure and a pointer to some
   member in the structure in the same fcb logical field table.
   For example, the following will not do what you suspect (BOO!):

    static FIELD TBL[] = {
      strgen(0,0,putmore,0,"T",0),
      strgen(1,getcode,putcode,st,"RU",""),	gets code,displays flags
      strgen(2,getstrn,putstrn,st.desc,0,0),
      strgen(3,0,putmore,0,"B",0)
    };
*/

/* If you were using something in struct fcb, too bad!  Find some other
   way to do what you want.  struct fcb is supposed to be hidden!
*/
typedef struct fcb FCB;

/* flags for an fcb */
#define FCBFLG_DEF	0x00	/* default flags */
#define FCBFLG_TOP	0x01	/* cursor goes to top on fcbfwd() */
#define FCBFLG_RDONLY	0x02	/* fcb is readonly, fcbupdate() will not */
				/* do anything, nor will fcbins() & fcbdel() */
#define FCBFLG_PHID	0x04	/* primary key is totally hidden, */
				/* fcbloc() will use the tag hold array */
#define FCBFLG_IGNERR	0x08	/* ignore update errors */
#define FCBFLG_REWRT	0x10	/* rewrite records on dupkeys */

typedef struct altfkey
{
  short fkey;			/* alternate function key */
  void (* func)( FCB * );	/* function called when key is pressed */
} ALTFKEY;
/* add these to fkey in ALTFKEY */
#define FCBKEY_NOUP	0x4000		/* key is a no update key */
#define FCBKEY_EXIT	0x2000		/* key is an exit key */

extern FCB *fcbopen( FIELD *, int tblsiz,
		       ALTFKEY *, int taglen, int tfld, int bfld,
		       PTR (*fcbio)(int ,const PTR ,PTR ),
		       PTR iocb, int linepos );
/* fcbopen is obsolete, it is included only for compatibility, you should
   use fcbbeg. The tfld and bfld parameters should now be entrys in your
   field table using the putmore function. Your code will now look like:
   ...
   static FCB *fcblist[2];
   static FIELD TBL[] = {
     strgen(TMARK,0,putmore,0,"T",0),
     strgen(NAME,getstrn,putstrn,st.name,"R",0),
     strgen(DESC,getstrn,putstrn,st.desc,0,0),
     strgen(BMARK,0,putmore,0,"B",0)
   };
   #define FCBT		 TBL
   #define FCBTSIZE	 count(TBL)

   main()
   {
     ...
     fcblist[0] = fcbbeg(FCBT,FCBTSIZE,fcbkeys,fileio,(PTR)fio,
			 FCBFLG_DEF+FCBFLG_RDONLY);
     ...
     fcbscrn(fcblist,count(TBL),TBL,nonfcbkeys,keyhelp);
     ...
   }
*/
extern FCB *fcbbeg( FIELD *, int, ALTFKEY *,
		    PTR (*fcbio)(int,const PTR,PTR ), PTR iocb, int);
extern void fcbend( FCB ** );
extern void fcbclose( FCB * );
extern int  fcbflags( FCB *, int, int );
extern int  fcbscrn( FCB **, int, FIELD *, ALTFKEY *, KEYLIST * );
extern void fcbsetdisp( FCB *, int, FIELD * );
extern int  fcblocate( FCB * , int );
extern void fcbloc( FCB * );
extern void fcbfwd( FCB * );
extern void fcblfwd( FCB * );
extern void fcbrev( FCB * );
extern void fcbnextln( FCB * );
extern void fcbdel( FCB * );
extern void fcbins( FCB * );
extern void fcbupdate( FCB * );
extern void display_fcb( FCB * );
extern void buf2sar( FCB * , int );
extern void sar2buf( FCB * , int );
extern void fcbdisplay( FCB * );
extern void fcberaser( FCB ** );
extern void fcberas( FCB * );
extern void fcbclear( FCB * );
extern FCB *fcbfind( int, FCB **);
extern void fcbnoop();
extern char *fcbinrec( FCB *, char * );
extern char *fcbdelrec( FCB *, char * );
extern PTR fcbrplrec(FCB *,int);
extern int putmore( int, char *, char *, int);

struct FCB_class {
  char *(*first)(FCB *);
  char *(*last)(FCB *);
  char *(*next)(FCB *);
  char *(*prev)(FCB *);
  char *(*locate)(FCB *,const char *);	/* locate tag */
  char *(*update)(FCB *,const char *);	/* update tag */
  char *(*add)(FCB *,const char *,int);	/* add before/any/after tag */
  char *(*del)(FCB *);		/* delete current record */
  void (*clear)(FCB *);
  char *(*get)(FCB *,const char *);	/* get for update */
  int (*taglen)(FCB *);
  void (*setnull)(FCB *);
  int (*isnull)(FCB *);
};

/* function codes passed to (*fcbio)() */
#define FCBFIRST	0	/* locate first record */
#define FCBLAST		1	/* locate last record */
#define FCBNEXT		2	/* locate next record */
#define FCBPREV		3	/* locate prev record */
#define FCBLOCATE	4	/* search for a record */
#define FCBUPDATE	5	/* update record */
#define FCBADD_BEFORE   6       /* add new record before current rec */
#define FCBADD_AFTER    7       /* add new record after current rec */
#define FCBDELETE	8	/* delete record */
#define FCBCLEAR        9       /* clear edit work array */
#define FCBDELCURR      10      /* delete current record */
#define FCBGET		11	/* get record for update */
#define FCBTAGLEN	12	/* returns tag (key) length */
#define FCBSETNULL	13	/* sets null key using current rec */
#define FCBISNULL	14	/* compares null key with current record */
				/* io function returns non-zero if null */
#define FCBADD		15	/* just add new record */

/* constants for fcbopen() (don't use fcbopen, use fcbbeg) */
#define FCBBOT		0	/* leave cursor on last line of table */
#define FCBTOP		1	/* leave cursor at top of table */

#endif
