#include <setjmp.h>

/* all objects initialized between BEGIN and END are destroyed
   when END is reached.  Returning out of a function before reaching
   END is unsafe (the stack location may be overwritten before the object
   is destroyed).  Using THROW or errpost() is safe.
*/
#define BEGIN	{ Object *__save = Object_head; {
#define END	  Object_clean(__save); } }

/* members and methods that must be in all objects that implement auto-
   destruction. */
typedef struct Object {
  struct Object_class {
    int size;
    void (*end)(/* struct Object * */);	/* uninitialize object */
    void (*del)(/**/ void *, int /**/);	/* deallocate object */
  } *_class;
  struct Object *_next;
} Object;

/* initialize allocated object

   The sequence of steps in an object initializer must be as follows:
   1)	initialize member objects	e.g. Foo_init(this->foobar);
   2)	initialize base object		e.g. Object_init(this);
   3)	assign class			e.g. this->base._class = &My_class;
   4)	private initialization
*/
extern void Object_init(void *);

extern void *Object_new(int);		/* allocate object */
extern void Object_del(void *,int);	/* deallocate object */
extern void Object_clean(Object *);	/* destroy all pending objects */
extern Object *Object_head;		/* chain of pending objects */

/* init function must allocate object if pointer is non-zero */

/* execute virtual functions */
#define do0(this,op)		(this)->_class->op(this)
#define do1(this,op,x)		(this)->_class->op(this,x)
#define do2(this,op,x,y)	(this)->_class->op(this,x,y)
#define do3(this,op,x,y,z)	(this)->_class->op(this,x,y,z)
#define do4(this,op,w,x,y,z)	(this)->_class->op(this,w,x,y,z)

/* reference class variables */
#define var(this,m)		(this)->_class->m

/* allocate initialized objects - always succeeds (or produces exception) */
extern void *push_new(void *(*)(int),int,void (*)(void *,int)), *pop_new(void),
	    *cnt_new(void (*)(),void *(*)(int),void (*)(void *,int),int,int);
#define new0(type)	\
	(type##_init(push_new(type##_new,sizeof(type),type##_del)),	\
		pop_new())
#define new1(type,p1)	\
	(type##_init(push_new(type##_new,sizeof(type),type##_del),p1), \
		pop_new())
#define new2(type,p1,p2)	\
	(type##_init(push_new(type##_new,sizeof(type),type##_del),p1,p2), \
		pop_new())
/* allocate an array of objects */
#define newa(type,n)	\
	cnt_new(type##_init,type##_new,type##_del,sizeof(type),n)

/* uninitialize & free objects allocated with new */
extern void deletea(void *,int);	/* delete an array of objects */
#define delete(p)	deletea(p,1)	/* delete a single object */

/* exception and object blocks */

typedef struct Object_Error {
  Object base;
  struct JMP_BUF *save;
  struct Object_Error *catch;
  jmp_buf mark;
} Object_Error;

/* CATCH is paired with END.  Exceptions posted between CATCH and
   END jump to END and destroy all objects initialized since CATCH.
   including objects created in nested functions.
*/

#define CATCH(rc)	{ Object *_save = Object_head;	\
			  Object_Error _Error;		\
			  Object_Error_init(&_Error);	\
			  if (!(rc = setjmp(_Error.mark))) {

#define THROW(i)	Object_Error_throw(i)

extern void Object_Error_init(Object_Error *);
extern volatile void Object_Error_throw(int);
